%{

/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef __string_H
#define __string_H
#include <string.h>
#endif
  
#ifndef __stdlib_H
#define __stdlib_H
#include <stdlib.h>
#endif

#ifndef __var_H
#include "var.h"
#endif

#ifndef __define_H
#include "define.h"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#ifndef __yooexcept_HXX
#include "yooexcept.hxx"
#endif

#include "repert.h"

/*****************************************************************************/

extern "C" {
  int yywrap() ;
  int yyparse() ;
}

#if defined(__alpha__) || (defined(__sun__) && !defined(__svr4__))
int yylex() ;
#endif

#if defined(__alpha__) || (defined(__sun__) && !defined(__svr4__))
int yyoutput(int c) ; 
#else
extern "C" { int yyoutput(int c) ; }
#endif

extern FILE* yyin ;

static ListVal<char*> l_bases ;

%}

%union  {
	int kwd;
	char text[256];
        } ;

%token <kwd> VOL SYSTEM SCHEMA CACHE CONFIG ALLOC INT WAITLOCK LOCKTIMEOUT
%token <kwd> BOOLEAN NUMTRANS
%token <text> IDENT DIR

%start init_list

%%

init_list : init
            | init init_list ;

init: vol system option_list

vol:
        VOL '.' IDENT ':' DIR
        {
	  l_bases += $3 ;
	  if (YOODA_BASE && (!strcmp($3, YOODA_BASE))) {
	    YOO_VOL_DIR = strdup($5) ;
	  }
        } ;

system:
        SYSTEM '.' IDENT ':' DIR
        { if (YOODA_BASE && (!strcmp($3, YOODA_BASE))) {
	    YOO_SYSTEM_DIR = strdup($5) ;
	  }
        }

option_list:
        /*empty */
        | option option_list ;
	
option :	
        SCHEMA '.' IDENT ':' DIR
        { if (YOODA_BASE && (!strcmp($3, YOODA_BASE))) {
	    YOO_SCH_DIR = strdup($5) ;
	  }
        }
        | CONFIG '.' IDENT '.' ALLOC ':' INT
	{ if (YOODA_BASE && (!strcmp($3, YOODA_BASE))) {
	    YOO_MAX_FREE_TO_ALLOC = $5 ;
	  } 
        }
        | CONFIG '.' IDENT '.' WAITLOCK ':' BOOLEAN
	{ if (YOODA_BASE && (!strcmp($3, YOODA_BASE))) {
	    YOO_NO_WAIT = !($5) ;
	  }
        }
        | CONFIG '.' IDENT '.' LOCKTIMEOUT ':' INT
	{ if (YOODA_BASE && (!strcmp($3, YOODA_BASE))) {
	    YOO_LOCK_TIMEOUT = $5 ;
	  }
        }
        | CONFIG '.' IDENT '.' NUMTRANS ':' INT
	{ if (YOODA_BASE && (!strcmp($3, YOODA_BASE))) {
	    YOO_NUM_TRANS = $5 ;
	  }
        }

%%

void yoo_repert() {

  char repert[512], vol[512] ;

  sprintf(repert, "%s/repert.sys", YOODA_HOME) ;
  SYSCALLP(yyin = fopen(repert, "r")) ;
  l_bases.clear() ;
  yyparse() ;
  fclose(yyin) ;

  if (YOO_SCH_DIR == NULL) {
    YOO_SCH_DIR = "." ;
  }
  
  if ((YOODA_BASE != NULL) && (YOO_VOL_DIR == NULL)) {
    char mess[512] ;
    sprintf(mess, "<%s> : Unknown database", YOODA_BASE) ;
    yoo_error(YOO_ERR_BASE, mess) ;
    exit(YOO_ERR_BASE) ;
  }
  if (YOODA_BASE != NULL) {
    sprintf(vol, "%s/%s_volume", YOO_SYSTEM_DIR, YOODA_BASE) ;
    YOO_VOL_FILE = strdup(vol) ;
  }
}


ListVal<char*>& list_bases() {

  YOODA_HOME = getenv("YOODA_HOME") ;
  if (YOODA_HOME == NULL) {
    YOODA_HOME = "." ;
  }
  yoo_repert() ;
  return l_bases ;
}


#ifdef hpux
void yyerror(char* s) {
#else
int yyerror(char* s) {
#endif
  perror("Error in repert.sys :") ;
  yoo_exit(1) ;
}
